import { addComponent, defineNuxtModule, createResolver, addPlugin, addImports } from '@nuxt/kit';
import { Components, Composables } from 'bootstrap-vue-next';

const useComponents = () => {
  Object.keys(Components).forEach((name) => {
    addComponent({
      name,
      export: name,
      filePath: "bootstrap-vue-next"
    });
  });
};

const parseActiveImports = (options, values) => {
  const { all, ...others } = options;
  const valuesCopy = {};
  if (all) {
    values.forEach((el) => {
      valuesCopy[el] = all;
    });
  }
  const merge = { ...valuesCopy, ...others };
  return Object.entries(merge).filter(([name, value]) => !!value && values.includes(name)).map(([name]) => name);
};

const module = defineNuxtModule({
  meta: {
    name: "bootstrap-vue-next",
    configKey: "bootstrapVueNext",
    compatibility: {
      nuxt: "^3.0.0",
      bridge: false
    }
  },
  defaults: {
    composables: true
  },
  setup(options, nuxt) {
    const { resolve } = createResolver(import.meta.url);
    addPlugin(resolve("./runtime/plugins/createBootstrap"));
    const normalizedComposableOptions = typeof options.composables === "boolean" ? { all: options.composables } : options.composables;
    nuxt.options.build.transpile.push(resolve("./runtime"));
    const transformAssetUrls = {
      BImg: ["src"]
    };
    if (nuxt.options.vite.vue === void 0) {
      nuxt.options.vite.vue = {
        template: {
          transformAssetUrls
        }
      };
    } else if (nuxt.options.vite.vue.template === void 0) {
      nuxt.options.vite.vue.template = {
        transformAssetUrls
      };
    } else if (nuxt.options.vite.vue.template.transformAssetUrls === void 0) {
      nuxt.options.vite.vue.template.transformAssetUrls = transformAssetUrls;
    } else if (
      // Do not overwrite user options
      !(typeof nuxt.options.vite.vue.template.transformAssetUrls !== "boolean" && ("BImg" in nuxt.options.vite.vue.template.transformAssetUrls || "b-img" in nuxt.options.vite.vue.template.transformAssetUrls))
    ) {
      Object.assign(nuxt.options.vite.vue.template.transformAssetUrls, transformAssetUrls);
    }
    nuxt.options.css.push("bootstrap-vue-next/dist/bootstrap-vue-next.css");
    nuxt.options.vite.optimizeDeps = nuxt.options.vite.optimizeDeps || {};
    nuxt.options.vite.optimizeDeps.include = nuxt.options.vite.optimizeDeps.include || [];
    nuxt.options.vite.optimizeDeps.include.push("bootstrap-vue-next");
    useComponents();
    if (Object.values(normalizedComposableOptions).some((el) => el === true)) {
      parseActiveImports(normalizedComposableOptions, Object.keys(Composables)).map(
        (name) => addImports({
          from: "bootstrap-vue-next",
          name
        })
      );
    }
  }
});

export { module as default };
